//
//  MyDocument.m
//  3DMixer
//
//  Created by Toshi Nagata on 08/02/08.
/*
    Copyright (c) 2008 Toshi Nagata. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "MyDocument.h"
#import "My3DMixer.h"

@implementation MyDocument

- (id)init
{
    self = [super init];
    if (self) {
		plist = nil;
    }
    return self;
}

- (NSString *)windowNibName
{
    return @"MyDocument";
}

- (void)newSettings
{
	id newset = [NSMutableDictionary dictionary];
	[newset setValue: [NSNumber numberWithFloat: 1.0] forKey: @"reference_distance"];
	[newset setValue: [NSNumber numberWithFloat: 50.0] forKey: @"maximum_distance"];
	[newset setValue: [NSNumber numberWithFloat: -10.0] forKey: @"maximum_attenuation"];
	[newset setValue: [NSNumber numberWithInt: 2] forKey: @"algorithm"];
	[newset setValue: [NSNumber numberWithInt: 0] forKey: @"channels"];
	[newset setValue: [NSNumber numberWithInt: 44100] forKey: @"bitrate"];
	
	[settingsController setContent: newset];
	[busesController setContent: nil];
}

- (void)windowControllerDidLoadNib:(NSWindowController *) aController
{
    [super windowControllerDidLoadNib:aController];
	if (plist == nil) {
		[self newSettings];
	} else {
		[settingsController setContent: [plist valueForKey: @"settings"]];
		[busesController setContent: [plist valueForKey: @"buses"]];
		[plist release];
		plist = nil;
	}
}

- (NSData *)dataRepresentationOfType:(NSString *)aType
{
	NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithObjectsAndKeys: [settingsController content], @"settings", [busesController content], @"buses", nil];
	return [NSPropertyListSerialization dataFromPropertyList: dict format: NSPropertyListXMLFormat_v1_0 errorDescription: NULL];
}

- (BOOL)loadDataRepresentation:(NSData *)data ofType:(NSString *)aType
{
	NSString *errorString;
	NSPropertyListFormat format;
	plist = [NSPropertyListSerialization propertyListFromData: data mutabilityOption: NSPropertyListMutableContainersAndLeaves format: &format errorDescription: &errorString];
	if (plist == nil) {
		[errorString release];
		return NO;
	}
	[plist retain];
	return YES;
}

#pragma mark ====== IB actions ======

- (IBAction)editSourceForSelection: (id)sender
{
	NSOpenPanel *openPanel = [NSOpenPanel openPanel];
	id obj;
	NSString *basename, *dirname, *newname;
	id selection = [busesController selectedObjects];
	if (selection == nil || [selection count] == 0)
		return;
	obj = [selection objectAtIndex: 0];
	basename = [obj valueForKey: @"source"];
	dirname = [obj valueForKey: @"source_dir"];
	[openPanel setAllowsMultipleSelection: NO];
	if ([openPanel runModalForDirectory: dirname file: basename types: nil] != NSOKButton)
		return;
	newname = [openPanel filename];
	basename = [newname lastPathComponent];
	dirname = [newname stringByDeletingLastPathComponent];
	[obj setValue: basename forKey: @"source"];
	[obj setValue: dirname forKey: @"source_dir"];
}

- (IBAction)addNewBus: (id)sender
{
	NSArray *ary;
	id obj;
	int i;
	NSOpenPanel *openPanel = [NSOpenPanel openPanel];
	[openPanel setAllowsMultipleSelection: YES];
	[openPanel setTreatsFilePackagesAsDirectories: YES];
	if ([openPanel runModalForTypes: nil] != NSOKButton)
		return;
	ary = [openPanel filenames];
	for (i = 0; i < [ary count]; i++) {
		id entry;
		NSString *basename, *dirname;
		obj = [ary objectAtIndex: i];
		entry = [NSMutableDictionary dictionary];
		basename = [obj lastPathComponent];
		dirname = [obj stringByDeletingLastPathComponent];
		[entry setValue: [NSNumber numberWithBool: YES] forKey: @"use"];
		[entry setValue: basename forKey: @"source"];
		[entry setValue: dirname forKey: @"source_dir"];
		[entry setValue: [NSNumber numberWithBool: YES] forKey: @"reverb"];
		[entry setValue: [NSNumber numberWithFloat: 0.0] forKey: @"azimuth"];
		[entry setValue: [NSNumber numberWithFloat: 0.0] forKey: @"elevation"];
		[entry setValue: [NSNumber numberWithFloat: 5.0] forKey: @"distance"];
		[entry setValue: [NSNumber numberWithFloat: 0.0] forKey: @"gain"];
		[entry setValue: [NSNumber numberWithFloat: 1.0] forKey: @"pitch"];
		[busesController addObject: entry];
	}
}

- (IBAction)startRendering: (id)sender
{
	NSSavePanel *savePanel = [NSSavePanel savePanel];
	[savePanel setAllowedFileTypes: [NSArray arrayWithObjects: @"aiff", nil]];
	if ([savePanel runModal] != NSFileHandlingPanelOKButton)
		return;
	[[[[My3DMixer alloc] init] autorelease] renderToFile: [savePanel filename] settings: [settingsController content] busSettings: [busesController content]];
}


@end
